#include "server.h"
#include "lion/icc/ICC.h"
#include "lion/rtos/RTOS_Sleep.h"
#include <argp.h>
#include <signal.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static const char *args_port;
static uint32_t    args_baudrate;
static const char *args_socket = "./lion-icc.sock";

static char args_doc[]    = "";
static char args_argdoc[] = "PORT [BAUDRATE]";

static struct argp_option args_options[] = {
    {"baud", 'b', "BAUDRATE", 0, "Specify the baudrate to use (Default: 115200)"},
    {"port", 'p', "PORT", 0, "Specify with tty device to use (Default: /dev/ttymxc1)"},
    {"socket", 's', "FILE", 0, "Specify a specific socket file to use. (Default: /var/run/lion-icc.sock)"},
    {"verbose", 'v', 0, 0, "Produce verbose output"},
    {0}
};

static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
    char *endptr;
    // printf("%d - \"%s\"\n ", key, arg);
    switch (key)
    {
        case 'p':
            break;
        case 'b':
            break;
        case 's':
            args_socket = arg;
            break;
        case ARGP_KEY_END:

            break;
        case ARGP_KEY_ARG:
            switch (state->arg_num)
            {
                case 0:
                    args_port = arg;
                    break;
                case 1:
                    args_baudrate = strtoul(arg, &endptr, 0);
                    if (args_baudrate == 0 || (endptr - arg) < strlen(arg))
                    {
                        argp_error(state, "'%s' is not a valid baudrate", arg);
                    }
                    break;
                default:
                    argp_usage(state);
            }
            break;
        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp args_argp = {args_options, parse_opt, args_argdoc, args_doc};

extern ICC_Context_t APP_strICC;

int main(int argc, char *argv[])
{
    argp_parse(&args_argp, argc, argv, 0, 0, 0);

    APP_ICC_Init();
    icc_server_init(NULL);
    APP_ICC_Start();
    icc_server_start();

    for (;;) {
        RTOS_Sleep(RTOS_SECONDS(3));
    }
    
    icc_server_stop();
    APP_ICC_Stop();

    return 0;
}