#include "../internal.h"
#include "../mcumgr.h"
#include "lion/log/log.h"
#include "lion/utils/byteorder.h"
#include "zcbor_decode.h"
#include <argp.h>
#include <assert.h>
#include <fcntl.h>
#include <openssl/sha.h>
#include <stdlib.h>

LOG_MODULE_REGISTER(image, LOG_LEVEL_DBG);

#define APP_IMG_LIST_FLAGS_BOOTABLE  ((uint32_t)1)
#define APP_IMG_LIST_FLAGS_ACTIVE    ((uint32_t)2)
#define APP_IMG_LIST_FLAGS_CONFIRMED ((uint32_t)4)
#define APP_IMG_LIST_FLAGS_PERMANENT ((uint32_t)8)
#define APP_IMG_LIST_FLAGS_PENDING   ((uint32_t)16)

static struct argp_option args_options[]        = {{0}};
static uint32_t           APP_IMG_Erase_u32Slot = 1;

static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
    char *endptr;
    switch (key)
    {
        case ARGP_KEY_END:
            break;
        case ARGP_KEY_ARG:
            switch (state->arg_num)
            {
                case 0:
                {
                    char *ptrEndPtr;
                    APP_IMG_Erase_u32Slot = (uint32_t)strtoul(arg, &ptrEndPtr, 10);
                    if ((ptrEndPtr - arg) != strlen(arg))
                    {
                        argp_error(state, "\"%s\" is not a valid integer", arg);
                    }
                    break;
                }
                default:
                    argp_usage(state);
            }
            break;
        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp args_argp = {args_options, parse_opt, ":SLOT", ""};

int APP_IMG_Erase_Main(int argc, char *argv[])
{
    int intRetVal = -1;
    argp_parse(&args_argp, argc, argv, 0, 0, 0);

    LOG_INF("Erasing Slot %u", APP_IMG_Erase_u32Slot);

    uint8_t       arrBuffer[2048];
    zcbor_state_t arrZCBORState[6] = {0};

    zcbor_new_encode_state(arrZCBORState, 6, arrBuffer, sizeof(arrBuffer), 1);

    assert(zcbor_map_start_encode(arrZCBORState, 0));
    assert(zcbor_tstr_put_lit(arrZCBORState, "slot"));
    assert(zcbor_uint32_encode(arrZCBORState, &APP_IMG_Erase_u32Slot));
    assert(zcbor_map_end_encode(arrZCBORState, 0));

    size_t   sizUsedLength     = (arrZCBORState[0].payload - arrBuffer);
    uint16_t u16ResponseLength = sizeof(arrBuffer);
    int intResult = app_mcumgr_Command(MCUMGR_OP_u8WRITE, MCUMGR_IMAGE_u16GROUP, MCUMGR_IMAGE_u8ID_ERASE, arrBuffer,
                                       sizUsedLength, arrBuffer, &u16ResponseLength, RTOS_SECONDS(12));

    if (intResult >= 0)
    {
        zcbor_new_decode_state(arrZCBORState, 4, arrBuffer, u16ResponseLength, 1);
        assert(zcbor_map_start_decode(arrZCBORState));
        struct zcbor_string strKey;
        uint32_t            u32Offset;
        uint32_t            u32RC;
        while (zcbor_tstr_decode(arrZCBORState, &strKey))
        {
            if (app_mcumgr_ParseRet(arrZCBORState, &strKey, &u32RC))
            {
                if (u32RC != 0U)
                {
                    LOG_ERR("Firmware reported error %u", u32RC);
                    exit(-1);
                }
            }
            else
            {
                char *szKey = malloc(strKey.len + 1);
                strncpy(szKey, strKey.value, strKey.len);
                LOG_WRN("Unknown key: \"%s\"", szKey);
                free(szKey);
                zcbor_any_skip(arrZCBORState, NULL);
            }
        }
        assert(zcbor_map_end_decode(arrZCBORState));
    }
    else
    {
        LOG_ERR("Failed to issue mcumgr command");
        intRetVal = -1;
    }

    return intRetVal;
}