#include "../internal.h"
#include "../mcumgr.h"
#include "COD_Compiler_Defines.h"
#include "lion/log/log.h"
#include "lion/utils/byteorder.h"
#include "zcbor_decode.h"
#include <argp.h>
#include <assert.h>
#include <fcntl.h>
#include <openssl/sha.h>
#include <stdlib.h>

LOG_MODULE_REGISTER(image, LOG_LEVEL_NONE);

static struct argp_option args_options[] = {{0}};

static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
    char *endptr;
    switch (key)
    {
        case ARGP_KEY_END:
            if (state->arg_num >= 1)
            {
                printf("Too many arguments!\n");
                argp_usage(state);
            }
            break;
        case ARGP_KEY_ARG:
            switch (state->arg_num)
            {
                default:
                    argp_usage(state);
            }
            break;
        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp args_argp = {args_options, parse_opt, "FILE", ""};

bool app_img_PrintImageList(zcbor_state_t *ptrZCBORState)
{
    assert(zcbor_list_start_decode(ptrZCBORState));
    while (zcbor_map_start_decode(ptrZCBORState))
    {
        StackScop uint32_t            u32_Slot  = UINT32_MAX;
        StackScop uint32_t            u32_Image = 0;
        StackScop char                sz_Key[64];
        StackScop char                sz_String[256] = "";
        StackScop struct zcbor_string str_Key;
        while (zcbor_tstr_decode(ptrZCBORState, &str_Key))
        {
            assert(str_Key.len <= sizeof(sz_Key));
            strncpy(sz_Key, str_Key.value, str_Key.len);
            sz_Key[str_Key.len] = '\0';

            if (strncmp("image", str_Key.value, str_Key.len) == 0)
            {
                assert(zcbor_uint32_decode(ptrZCBORState, &u32_Image));
            }
            else if (strncmp("slot", str_Key.value, str_Key.len) == 0)
            {
                assert(zcbor_uint32_decode(ptrZCBORState, &u32_Slot));
            }
            else
            {
                StackScop bool                bol_Boolean;
                StackScop struct zcbor_string str_String;
                if (zcbor_tstr_decode(ptrZCBORState, &str_String))
                {
                    strncpy(sz_String, str_String.value, str_String.len);
                    sz_String[str_String.len] = '\0';
                    printf("%d:%d:%s:%s\n", u32_Image, u32_Slot, sz_Key, sz_String);
                }
                else if (zcbor_bstr_decode(ptrZCBORState, &str_String))
                {
                    printf("%d:%d:%s:", u32_Image, u32_Slot, sz_Key);
                    for (uint8_t u8Index = 0; u8Index < str_String.len; u8Index++)
                    {
                        assert(printf("%02x", str_String.value[u8Index]) == 2);
                    }
                    printf("\n");
                }
                else if (zcbor_bool_decode(ptrZCBORState, &bol_Boolean))
                {
                    printf("%d:%d:%s:%s\n", u32_Image, u32_Slot, sz_Key, bol_Boolean ? "true" : "false");
                }
                else
                {
                    printf("%d:%d:%s:unknown\n", u32_Image, u32_Slot, sz_Key);
                    assert(zcbor_any_skip(ptrZCBORState, NULL));
                }
            }
        }
        assert(zcbor_map_end_decode(ptrZCBORState));
    }
    assert(zcbor_list_end_decode(ptrZCBORState));

    return true;
}

int APP_IMG_List_Main(int argc, char *argv[])
{
    int intRetVal = -1;
    argp_parse(&args_argp, argc, argv, 0, 0, 0);

    uint8_t       arrBuffer[2040 - sizeof(MCUMGR_Header_t)];
    zcbor_state_t arrZCBORState[6] = {0};

    zcbor_new_encode_state(arrZCBORState, 6, arrBuffer, sizeof(arrBuffer), 1);

    assert(zcbor_map_start_encode(arrZCBORState, 0));
    assert(zcbor_map_end_encode(arrZCBORState, 0));

    size_t   sizUsedLength     = (arrZCBORState[0].payload - arrBuffer);
    uint16_t u16ResponseLength = sizeof(arrBuffer);
    int      intResult = app_mcumgr_Command(MCUMGR_OP_u8READ, MCUMGR_IMAGE_u16GROUP, MCUMGR_IMAGE_u8ID_STATE, arrBuffer,
                                            sizUsedLength, arrBuffer, &u16ResponseLength, RTOS_SECONDS(5));
    if (intResult < 0)
    {
        printf("MCUMGR Execution Error %d \"%s\"", intResult, strerror(-intResult));
        return EIO;
    }

    StackScop uint32_t u32_RC = 0;
    zcbor_new_decode_state(arrZCBORState, 6, arrBuffer, sizeof(arrBuffer), 1);

    assert(zcbor_map_start_decode(arrZCBORState));
    struct zcbor_string strKey;
    while (zcbor_tstr_decode(arrZCBORState, &strKey))
    {
        if (strncmp("images", strKey.value, strKey.len) == 0)
        {
            assert(app_img_PrintImageList(arrZCBORState));
        }
        else if (app_mcumgr_ParseRet(arrZCBORState, &strKey, &u32_RC))
        {
        }
        else
        {
            assert(zcbor_any_skip(arrZCBORState, NULL));
        }
    }
    assert(zcbor_map_end_decode(arrZCBORState));

    return u32_RC;
}