#include "../internal.h"
#include "../mcumgr.h"
#include "COD_Compiler_Defines.h"
#include "img_mgmt.h"
#include "lion/log/log.h"
#include "lion/utils/UTIL.h"
#include "lion/utils/byteorder.h"
#include "zcbor_decode.h"
#include <argp.h>
#include <assert.h>
#include <fcntl.h>
#include <openssl/sha.h>
#include <stdlib.h>

LOG_MODULE_REGISTER(image, LOG_LEVEL_NONE);

static uint8_t APP_IMG_Test_arrImageHash[256];
static size_t  APP_IMG_Test_sizImageHashLength;

static struct argp_option args_options[] = {{0}};

static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
    char *endptr;
    switch (key)
    {
        case ARGP_KEY_INIT:
            APP_IMG_Test_sizImageHashLength = 0;
            break;
        case ARGP_KEY_END:
            if (state->arg_num < 1)
            {
                printf("Too few arguments!\n");
                argp_usage(state);
            }
            break;
        case ARGP_KEY_ARG:
            switch (state->arg_num)
            {
                case 0:
                    APP_IMG_Test_sizImageHashLength =
                        UTIL_Hex2Bin(arg, strlen(arg), APP_IMG_Test_arrImageHash, sizeof(APP_IMG_Test_arrImageHash));
                    if (APP_IMG_Test_sizImageHashLength <= 0)
                    {
                        argp_error(state, "\"%s\" isn't a valid hex string", arg);
                    }
                    break;
                default:
                    argp_usage(state);
            }
            break;
        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp args_argp = {args_options, parse_opt, "hex-image-hash", ""};

int APP_IMG_Test_Main(int argc, char *argv[])
{
    int intRetVal = -1;
    argp_parse(&args_argp, argc, argv, 0, 0, 0);

    uint8_t       arrBuffer[2048];
    zcbor_state_t arrZCBORState[6] = {0};
    zcbor_new_encode_state(arrZCBORState, 6, arrBuffer, sizeof(arrBuffer), 1);

    assert(zcbor_map_start_encode(arrZCBORState, 0));
    assert(APP_IMG_Test_sizImageHashLength > 0);
    assert(zcbor_tstr_put_lit(arrZCBORState, "hash"));
    assert(zcbor_bstr_encode_ptr(arrZCBORState, APP_IMG_Test_arrImageHash, APP_IMG_Test_sizImageHashLength));
    assert(zcbor_tstr_put_lit(arrZCBORState, "confirm"));
    assert(zcbor_bool_put(arrZCBORState, false));
    assert(zcbor_map_end_encode(arrZCBORState, 0));

    size_t   sizUsedLength     = (arrZCBORState[0].payload - arrBuffer);
    uint16_t u16ResponseLength = sizeof(arrBuffer);
    int intResult = app_mcumgr_Command(MCUMGR_OP_u8WRITE, MCUMGR_IMAGE_u16GROUP, MCUMGR_IMAGE_u8ID_STATE, arrBuffer,
                                       sizUsedLength, arrBuffer, &u16ResponseLength, RTOS_SECONDS(5));
    if (intResult < 0)
    {
        printf("MCUMGR Execution Error %d \"%s\"", intResult, strerror(-intResult));
        return EIO;
    }

    StackScop uint32_t u32_RC = 0;
    zcbor_new_decode_state(arrZCBORState, 6, arrBuffer, sizeof(arrBuffer), 1);

    assert(zcbor_map_start_decode(arrZCBORState));
    struct zcbor_string strKey;
    while (zcbor_tstr_decode(arrZCBORState, &strKey))
    {
        if (strncmp("images", strKey.value, strKey.len) == 0)
        {
            assert(app_img_PrintImageList(arrZCBORState));
        }
        else if (app_mcumgr_ParseRet(arrZCBORState, &strKey, &u32_RC))
        {
        }
        else
        {
            assert(zcbor_any_skip(arrZCBORState, NULL));
        }
    }
    assert(zcbor_map_end_decode(arrZCBORState));

    return u32_RC;
}