#include "../internal.h"
#include "../mcumgr.h"
#include "COD_Compiler_Defines.h"
#include "lion/log/log.h"
#include "zcbor_decode.h"
#include "zcbor_encode.h"
#include <argp.h>
#include <stdio.h>
#include <stdlib.h>

LOG_MODULE_REGISTER(os, LOG_LEVEL_NONE);

FileScop bool APP_IMG_Test_bol_Force = false;

static struct argp_option args_options[] = {
    {.name = "force", .key = 'f', .arg = NULL, .flags = 0, .doc = "Forced Reset", .group = 0},
    {0}
};

static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
    switch (key)
    {
        case 'f':
            APP_IMG_Test_bol_Force = true;
            break;
        case ARGP_KEY_INIT:
            break;
        case ARGP_KEY_END:
            break;
        case ARGP_KEY_ARG:
            break;
        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp args_argp = {args_options, parse_opt, "", ""};

int APP_OS_Reset_Main(int argc, char *argv[])
{
    argp_parse(&args_argp, argc, argv, ARGP_IN_ORDER, NULL, NULL);

    StackScop uint8_t arr_u8_Request[16];
    StackScop         ZCBOR_STATE_E(arr_EncodeState, 1, arr_u8_Request, sizeof(arr_u8_Request), 1);

    if (APP_IMG_Test_bol_Force)
    {
        assert(zcbor_map_start_encode(arr_EncodeState, 1));
        assert(zcbor_tstr_put_lit(arr_EncodeState, "force"));
        assert(zcbor_bool_put(arr_EncodeState, true));
        assert(zcbor_map_end_encode(arr_EncodeState, 1));
    }
    StackScop uint16_t u16_RequestLength = (uint16_t)(arr_EncodeState[0].payload - (uint8_t *)arr_u8_Request);

    StackScop uint8_t  arr_u8_Response[256];
    StackScop uint16_t u16_ResponseLength = sizeof(arr_u8_Response);
    StackScop int      int_res =
        app_mcumgr_Command(MCUMGR_OP_u8WRITE, MCUMGR_OS_u16GROUP, MCUMGR_OS_u8ID_RESET, arr_u8_Request,
                           u16_RequestLength, arr_u8_Response, &u16_ResponseLength, RTOS_SECONDS(15));
    if (int_res < 0)
    {
        printf("MCUMGR Execution Error %d \"%s\"", int_res, strerror(-int_res));
        return EIO;
    }

    StackScop ZCBOR_STATE_D(arr_DecodeState, 1, arr_u8_Response, u16_ResponseLength, 1);
    assert(zcbor_map_start_decode(arr_DecodeState));
    StackScop struct zcbor_string str_Key;
    StackScop uint32_t            u32_RC = 0;
    while (zcbor_tstr_decode(arr_DecodeState, &str_Key))
    {
        if (app_mcumgr_ParseRet(arr_DecodeState, &str_Key, &u32_RC))
        {
        }
        else
        {
            char *sz_Key = malloc(str_Key.len + 1);
            strncpy(sz_Key, str_Key.value, str_Key.len);
            LOG_WRN("Unknown key: \"%s\"", sz_Key);
            free(sz_Key);
            zcbor_any_skip(arr_DecodeState, NULL);
        }
    }
    assert(zcbor_map_end_decode(arr_DecodeState));

    if (u32_RC == 0)
    {
        printf("Device successfully %sreset!\n", APP_IMG_Test_bol_Force ? "forced " : "");
    }
    else
    {
        printf("Device unable to %sreset! (%u)\n", APP_IMG_Test_bol_Force ? "force " : "", u32_RC);
    }

    return u32_RC;
}