#ifndef LION_UTILS_UTIL_H__
#define LION_UTILS_UTIL_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

#include <stdint.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C"
{
#endif

    /* ======================================================================= */
    /* ============================ T Y P E D E F S ========================== */
    /* ======================================================================= */

    /* ======================================================================= */
    /* ============================= D E F I N E S =========================== */
    /* ======================================================================= */

// #define UTIL_ARGS_UNUSED(...) UTIL_ARG_UNUSED(__VA_ARGS__)
// #define UTIL_ARG_UNUSED(arg, ...) (void)(arg); UTIL_ARG_UNUSED(__VA_ARGS__)
// #define UTIL_ARG_UNUSED()


// #define util_impl_MAX(type, a, b) ({\
//     type _a = (a); \
//     type _b = (b); \
//     _a > _b ? _a : _b; \
// })

// #define util_impl_MAXV(type, a, ...) a + util_impl_MAXV(type, __VA_ARGS__)

// #define UTIL_MAX(type, ...) util_impl_MAXV(type, __VA_ARGS__)

    /* ======================================================================= */
    /* ========================== P R O T O T Y P E S ======================== */
    /* ======================================================================= */
    
/**
 * @brief      Convert a single character into a hexadecimal nibble.
 *
 * @param c     The character to convert
 * @param x     The address of storage for the converted number.
 *
 *  @return Zero on success or (negative) error code otherwise.
 */
int UTIL_Char2Hex(char c, uint8_t *x);

/**
 * @brief      Convert a single hexadecimal nibble into a character.
 *
 * @param c     The number to convert
 * @param x     The address of storage for the converted character.
 *
 *  @return Zero on success or (negative) error code otherwise.
 */
int UTIL_Hex2Char(uint8_t x, char *c);

/**
 * @brief      Convert a binary array into string representation.
 *
 * @param buf     The binary array to convert
 * @param buflen  The length of the binary array to convert
 * @param hex     Address of where to store the string representation.
 * @param hexlen  Size of the storage area for string representation.
 *
 * @return     The length of the converted string, or 0 if an error occurred.
 */
size_t UTIL_Bin2Hex(const uint8_t *buf, size_t buflen, char *hex, size_t hexlen);

/**
 * @brief      Convert a hexadecimal string into a binary array.
 *
 * @param hex     The hexadecimal string to convert
 * @param hexlen  The length of the hexadecimal string to convert.
 * @param buf     Address of where to store the binary data
 * @param buflen  Size of the storage area for binary data
 *
 * @return     The length of the binary array, or 0 if an error occurred.
 */
size_t UTIL_Hex2Bin(const char *hex, size_t hexlen, uint8_t *buf, size_t buflen);

#ifdef __cplusplus
}
#endif

#endif /* LION_UTILS_UTIL_H__ */