#ifndef LION_ICC_TRANSPORT_H__
#define LION_ICC_TRANSPORT_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

#include "lion/utils/dlist.h"
#include <stddef.h>
#include <stdint.h>
#include "COD_Compiler_Defines.h"

#ifdef __cplusplus
extern "C"
{
#endif

    /* ======================================================================= */
    /* ============================= D E F I N E S =========================== */
    /* ======================================================================= */

    /* ======================================================================= */
    /* ============================ T Y P E D E F S ========================== */
    /* ======================================================================= */

    typedef struct ICC_Context_t ICC_Context_t;
    typedef struct ICC_Packet_t  ICC_Packet_t;

    typedef void (*ICC_Transport_Process_t)(ICC_Context_t *iccContext, void *userData);
    typedef void (*ICC_Transport_Reset_t)(ICC_Context_t *iccContext, void *userData);

    struct ICC_Transport_t
    {
        sys_dnode_t             node;
        ICC_Context_t          *context;
        ICC_Transport_Process_t process;
        ICC_Transport_Reset_t   reset;
        void                   *userData;
        ICC_Packet_t           *ptrRXPacket;
        sys_dlist_t             lst_TXPackets;
    };
    typedef struct ICC_Transport_t ICC_Transport_t;

    /* ======================================================================= */
    /* ========================== P R O T O T Y P E S ======================== */
    /* ======================================================================= */

    void ICC_Transport_Init(ICC_Transport_t *transport, ICC_Transport_Process_t process, ICC_Transport_Reset_t reset, void *userData);

    int ICC_AddTransport(ICC_Context_t *context, ICC_Transport_t *transport);
    int ICC_RemoveTransport(ICC_Context_t *context, ICC_Transport_t *transport);
    
/**
 * @brief Release a popped TX packet back to the ICC to be enqueued to the next transport.
 * @warning Must ONLY be used as part of the implementation for a transport.
*/
GlobScop void ICC_Transport_ReleaseTXPacket(ICC_Transport_t *ptr_Transport, ICC_Packet_t *ptr_Packet);

/**
 * @brief Retrieve the next TX packet enqueued to the specified transport.
 * @warning Must ONLY be used as part of the implementation for a transport.
 * @warning EVERY call to ICC_Transport_PopTXPacket MUST be followed by a call to ICC_Transport_ReleaseTXPacket.
 */
GlobScop ICC_Packet_t *ICC_Transport_PopTXPacket(ICC_Transport_t *ptr_Transport);

#ifdef __cplusplus
}
#endif

#endif /* LION_ICC_TRANSPORT_H__ */