#ifndef ICC_TRANSPORT_STREAM_H__
#define ICC_TRANSPORT_STREAM_H__

#include <lion/icc/ICC_Transport.h>
#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef CONFIG_ICC_STREAM_DELIMITER
    #define CONFIG_ICC_STREAM_DELIMITER 0xAA, 0x55
#endif

    /**********************************************************************************************************************
     * @cond                                             GLOBAL SCOPE @endcond
     *********************************************************************************************************************/
    
    typedef enum {
        ICC_STREAM_RX_WAIT_FOR_SYNC,
        ICC_STREAM_RX_GET_LENGTH,
        ICC_STREAM_RX_ALLOCATE_PACKET,
        ICC_STREAM_RX_GET_PACKET,
        ICC_STREAM_RX_GET_CRC
    } ICC_Stream_RX_State_t;

    typedef enum {
        ICC_STREAM_TX_IDLE,
        ICC_STREAM_TX_PUT_SYNC,
        ICC_STREAM_TX_PUT_LENGTH,
        ICC_STREAM_TX_PUT_PACKET,
        ICC_STREAM_TX_PUT_CRC
    } ICC_Stream_TX_State_t;

    /**
     * @brief Called to write data to the stream
     * @param ptr_u8_Data Pointer to data to be written
     * @param siz_Length Requested number of bytes to write
     * @param bol_All When True, data should only be written if all of it can be written at once
     * @param ptr_UserData User specified pointer provide via @ref ICC_Stream_Init
     * @returns Number of bytes written
     */
    typedef size_t (*ICC_Stream_Write_t)(const uint8_t *ptr_u8_Data, size_t siz_Length, bool bol_All,
                                         void *ptr_UserData);

    /**
     * @brief Called to read data from the stream
     * @param ptr_u8_Data Pointer to data buffer for reading
     * @param siz_MaxLength Size of data buffer, and requested bytes to read
     * @param bol_All When True, data should only be read if all requeested bytes can be read
     * @param ptr_UserData User specified pointer provide via @ref ICC_Stream_Init
     * @returns Number of bytes copied to @ref ptr_u8_Data
     */
    typedef size_t (*ICC_Stream_Read_t)(uint8_t *ptr_u8_Data, size_t siz_MaxLength, bool bol_All, void *ptr_UserData);

    typedef struct ICC_Stream_Context_t
    {
        ICC_Transport_t transport;
        uint16_t        u16EmptyRXCycles; /* The number of times RX has been processed without advancing. */

        uint16_t            u16_TX_Cycles;

        ICC_Stream_RX_State_t rx_state;
        uint16_t              rx_frameCounter;
        uint16_t              rx_frameLength;
        uint16_t              rx_calculatedCRC;
        ICC_Packet_t         *rx_packet;

        ICC_Stream_TX_State_t tx_state;
        uint16_t              tx_calculatedCRC;
        ICC_Packet_t         *tx_packet;
        uint16_t              tx_frameCounter;
        uint16_t              tx_frameLength;

        ICC_Stream_Write_t fnWrite;
        ICC_Stream_Read_t  fnRead;
        void              *ptrUserData;
    } ICC_Stream_Context_t;

    void ICC_Stream_Init(ICC_Stream_Context_t *context, ICC_Stream_Write_t fnWrite, ICC_Stream_Read_t fnRead,
                         void *ptrUserData);

#ifdef __cplusplus
}
#endif

#endif /* ICC_TRANSPORT_STREAM_H__ */