#ifndef ICC_CLIENT_H__
#define ICC_CLIENT_H__

#include "lion/icc/ICC.h"
#include "lion/icc/transport/ICC_Stream.h"
#include "lion/rtos/RTOS_CondVar.h"
#include <stdbool.h>

#ifdef __cplusplus
extern "C"
{
#endif

    typedef enum ICC_Client_State_t {
        ICC_CLIENT_DISABLED,
        ICC_CLIENT_DISCONNECTED,
        ICC_CLIENT_CONNECTED
    } ICC_Client_State_t;

    typedef struct ICC_Client_Context_t
    {
        pthread_t            fdThread;
        ICC_Context_t       *ptrICC;
        ICC_Stream_Context_t strStream;
        volatile bool        bolBroken;
        volatile bool        bolKeepRunning;
        const char          *szSocketPath;
        int                  fdSocket;
        ICC_Client_State_t   enmState;
        RTOS_Mutex_t         strStateLock;
        RTOS_CondVar_t       strStateChanged;

    } ICC_Client_Context_t;

    void ICC_Client_Init(ICC_Client_Context_t *ptrContext, ICC_Context_t *ptrICC, const char *szSocketPath);
    int  ICC_Client_Start(ICC_Client_Context_t *ptrContext);
    int  ICC_Client_Stop(ICC_Client_Context_t *ptrContext);

    ICC_Client_State_t ICC_Client_GetState(ICC_Client_Context_t *ptrContext);
    int ICC_Client_WaitForState(ICC_Client_Context_t *ptrContext, ICC_Client_State_t enmState, RTOS_Timeout_t strTimeout);

    static inline int ICC_Client_WaitForConnected(ICC_Client_Context_t *ptrContext, RTOS_Timeout_t strTimeout) {
        return ICC_Client_WaitForState(ptrContext, ICC_CLIENT_CONNECTED, strTimeout);
    }

#ifdef __cplusplus
}
#endif

#endif /* ICC_CLIENT_H__ */